blockquote {
    font-family: "Courier New", Courier, monospace;
    background-color: #FFEEFF;
}

.copyright {
    text-align: right;
    font-size: 0.8em;
    color: #999999;
}

.copyright > a {
    color: #999999;
}
